@@ -41,6 +41,7 @@ typedef struct {
 
   GtkWidget *entry;
   GtkWidget *ticky;
+  GtkWidget *ok_button;
 
   gboolean grabbing;
 
@@ -135,12 +136,16 @@ static void
 password_entry_changed_cb (GtkEditable *entry,
     gpointer user_data)
 {
+  EmpathyPasswordDialogPriv *priv = EMPATHY_PASSWORD_DIALOG (user_data)->priv;
   const gchar *str;
 
   str = gtk_entry_get_text (GTK_ENTRY (entry));
 
   gtk_entry_set_icon_sensitive (GTK_ENTRY (entry),
       GTK_ENTRY_ICON_SECONDARY, !EMP_STR_EMPTY (str));
+
+  gtk_widget_set_sensitive (priv->ok_button,
+      !EMP_STR_EMPTY (str));
 }
 
 static gboolean
@@ -202,6 +207,13 @@ password_dialog_window_state_changed (GtkWidget *widget,
   return FALSE;
 }
 
+static void
+password_dialog_handler_invalidated_cb (EmpathyServerSASLHandler *handler,
+    EmpathyPasswordDialog *dialog)
+{
+  gtk_widget_destroy (GTK_WIDGET (dialog));
+}
+
 static void
 empathy_password_dialog_constructed (GObject *object)
 {
@@ -221,11 +233,17 @@ empathy_password_dialog_constructed (GObject *object)
 
   account = empathy_server_sasl_handler_get_account (priv->handler);
 
+  tp_g_signal_connect_object (priv->handler, "invalidated",
+      G_CALLBACK (password_dialog_handler_invalidated_cb),
+      object, 0);
+
   /* dialog */
-  gtk_dialog_add_buttons (GTK_DIALOG (dialog),
-      GTK_STOCK_OK, GTK_RESPONSE_OK,
-      GTK_STOCK_CANCEL, GTK_RESPONSE_CANCEL,
-      NULL);
+  gtk_dialog_add_button (GTK_DIALOG (dialog),
+      GTK_STOCK_CANCEL, GTK_RESPONSE_CANCEL);
+
+  priv->ok_button = gtk_dialog_add_button (GTK_DIALOG (dialog),
+      GTK_STOCK_OK, GTK_RESPONSE_OK);
+  gtk_widget_set_sensitive (priv->ok_button, FALSE);
 
   text = g_strdup_printf (_("Enter your password for account\n<b>%s</b>"),
       tp_account_get_display_name (account));
@@ -256,7 +274,7 @@ empathy_password_dialog_constructed (GObject *object)
   g_signal_connect (priv->entry, "icon-release",
       G_CALLBACK (clear_icon_released_cb), NULL);
   g_signal_connect (priv->entry, "changed",
-      G_CALLBACK (password_entry_changed_cb), NULL);
+      G_CALLBACK (password_entry_changed_cb), dialog);
 
   gtk_box_pack_start (box, priv->entry, FALSE, FALSE, 0);
   gtk_widget_show (priv->entry);
@@ -264,15 +282,8 @@ empathy_password_dialog_constructed (GObject *object)
   /* remember password ticky box */
   priv->ticky = gtk_check_button_new_with_label (_("Remember password"));
 
-  /* Don't add this to the dialog yet because we haven't set up
-   * everything in the UI properly yet and the MC transition isn't
-   * ready etc. so we'll just force it to never remember a
-   * password. */
-  gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (priv->ticky), FALSE);
-  /*
   gtk_box_pack_start (box, priv->ticky, FALSE, FALSE, 0);
   gtk_widget_show (priv->ticky);
-  */
 
   g_signal_connect (dialog, "response",
       G_CALLBACK (password_dialog_response_cb), dialog);
